package com.mygdx.game;

import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.g3d.Material;
import com.badlogic.gdx.graphics.g3d.ModelBatch;
import com.badlogic.gdx.graphics.g3d.attributes.BlendingAttribute;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.utils.ScreenUtils;
import com.mbrlabs.mundus.commons.Scene;
import com.mbrlabs.mundus.commons.event.BeforeDeptRenderEvent;
import com.mbrlabs.mundus.commons.event.BeforeRenderEvent;
import com.mbrlabs.mundus.commons.scene3d.GameObject;
import com.mbrlabs.mundus.commons.scene3d.components.ModelComponent;
import com.mbrlabs.mundus.runtime.Mundus;

public class MyGdxGame extends ApplicationAdapter {
	private ModelBatch batch;

	private Mundus mundus;
	private Scene scene;

	private TopDownCameraController controller;

	private BlendingAttribute blendingAttribute;

	private boolean show = true;

	@Override
	public void create () {
		batch = new ModelBatch();

		mundus = new Mundus(Gdx.files.internal("mundus"));
		scene = mundus.loadScene("Main Scene.mundus");

		scene.cam.position.set(320, 50, 450);
		scene.cam.up.add(0, 50, 450);
		scene.cam.direction.rotate(Vector3.X, 90.0f);

		controller = new TopDownCameraController(scene.cam);
		Gdx.input.setInputProcessor(controller);


		scene.sceneGraph.getRoot().getChildren().get(1).getComponents();
		for (final GameObject child : scene.sceneGraph.getRoot().getChildren()) {
			if (child.name.equals("house.gltf")) {
				final ModelComponent modelComponent = (ModelComponent) child.getComponents().get(0);

				blendingAttribute = new BlendingAttribute();

				final Material material = modelComponent.getModelInstance().getMaterial("Material.002");
				material.set(blendingAttribute);

				modelComponent.addEvent(new BeforeRenderEvent() {
					@Override
					public void action() {
						if (!show) {
							blendingAttribute.opacity = 0.0f;
						}
					}
				});

				modelComponent.addEvent(new BeforeDeptRenderEvent() {
					@Override
					public void action() {
						blendingAttribute.opacity = 1.0f;
					}
				});

			}
		}
	}

	@Override
	public void render () {
		ScreenUtils.clear(1, 0, 0, 1);

		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT | GL20.GL_DEPTH_BUFFER_BIT);

		if (Gdx.input.isKeyJustPressed(Input.Keys.R)) {
			show = !show;
		}

		controller.update(Gdx.graphics.getDeltaTime());
		scene.sceneGraph.update();
		scene.render();
	}

	@Override
	public void dispose () {
		mundus.dispose();
		batch.dispose();
	}
}
