package com.mygdx.game;

import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Camera;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.utils.IntIntMap;

public class TopDownCameraController extends InputAdapter {

    private static final float VELOCITY = 200f;

    private final Camera camera;

    private final IntIntMap keys;

    public TopDownCameraController(final Camera camera) {
        this.camera = camera;
        keys = new IntIntMap();
    }

    @Override
    public boolean keyDown (final int keycode) {
        keys.put(keycode, keycode);
        return true;
    }

    @Override
    public boolean keyUp (final int keycode) {
        keys.remove(keycode, 0);
        return true;
    }

    public void update(final float deltaTime) {
        if (keys.containsKey(Input.Keys.W)) {
            camera.position.add(0, 0, deltaTime * VELOCITY);
        }
        if (keys.containsKey(Input.Keys.S)) {
            camera.position.add(0, 0, -deltaTime * VELOCITY);
        }
        if (keys.containsKey(Input.Keys.A)) {
            camera.position.add(deltaTime * VELOCITY, 0, 0);
        }
        if (keys.containsKey(Input.Keys.D)) {
            camera.position.add(-deltaTime * VELOCITY, 0, 0);
        }
        if (keys.containsKey(Input.Keys.Q)) {
            camera.direction.rotate(Vector3.X, 1.0f);
        }
        if (keys.containsKey(Input.Keys.E)) {
            camera.direction.rotate(Vector3.X, -1.0f);
        }
        camera.update(true);
    }
}
